package eu.jonahbauer.wizard.client.libgdx.state;

import eu.jonahbauer.wizard.client.libgdx.Client;
import eu.jonahbauer.wizard.client.libgdx.screens.CreateGameScreen;
import eu.jonahbauer.wizard.client.libgdx.screens.LobbyScreen;
import eu.jonahbauer.wizard.common.messages.client.CreateSessionMessage;
import eu.jonahbauer.wizard.common.messages.client.JoinSessionMessage;
import eu.jonahbauer.wizard.common.messages.data.SessionData;
import eu.jonahbauer.wizard.common.messages.server.*;
import eu.jonahbauer.wizard.common.model.Configuration;
import lombok.Getter;
import lombok.Setter;

import java.util.*;

public final class Lobby extends BaseState {

    private final Map<UUID, SessionData> sessions = new HashMap<>();

    private LobbyScreen lobbyScreen;

    @Getter
    @Setter
    private String playerName = "";
    
    public Lobby(SessionListMessage list) {
        list.getSessions().forEach(s -> sessions.put(s.getUuid(), s));
    }
    
    @Override
    public Optional<ClientState> onEnter(Client client) {
        showListScreen(client);
        return super.onEnter(client);
    }
    
    @Override
    public Optional<ClientState> onMessage(Client client, ServerMessage message) {
        if (message instanceof SessionCreatedMessage created) {
            var session = created.getSession();
            sessions.put(session.getUuid(), session);
            if (lobbyScreen != null) {
                lobbyScreen.addSession(session);
            }
            return Optional.empty();
        } else if (message instanceof SessionRemovedMessage removed) {
            var session = removed.getSession();
            sessions.remove(session);
            if (lobbyScreen != null) {
                lobbyScreen.removeSession(session);
            }
            return Optional.empty();
        } else if (message instanceof SessionModifiedMessage modified) {
            var session = modified.getSession();
            sessions.put(session.getUuid(), session);
            if (lobbyScreen != null) {
                lobbyScreen.modifySession(session);
            }
            return Optional.empty();
        } else if (message instanceof SessionListMessage list) {
            list.getSessions().forEach(s -> sessions.put(s.getUuid(), s));
            if (lobbyScreen != null) {
                lobbyScreen.setSessions(list.getSessions().toArray(new SessionData[0]));
            }
            return Optional.empty();
        } else {
            return unexpectedMessage(client, message);
        }
    }

    public Optional<ClientState> disconnect(@SuppressWarnings("unused") Client client) {
        return Optional.of(new Menu());
    }

    public Optional<ClientState> createSession(Client client, String sessionName, Configuration config, long timeout, String playerName) {
        client.send(new CreateSessionMessage(sessionName, playerName, timeout, config));
        return Optional.of(new AwaitingJoinSession(null, sessionName, config, playerName));
    }

    public Optional<ClientState> joinSession(Client client, SessionData session, String playerName) {
        client.send(new JoinSessionMessage(session.getUuid(), playerName));
        return Optional.of(new AwaitingJoinSession(session.getUuid(), session.getName(), session.getConfiguration(), playerName));
    }

    public Optional<ClientState> showCreateScreen(Client client) {
        var game = client.getGame();
        lobbyScreen = null;
        game.setScreen(new CreateGameScreen(game, playerName));
        return Optional.empty();
    }

    public Optional<ClientState> showListScreen(Client client) {
        var game = client.getGame();
        lobbyScreen = new LobbyScreen(game, playerName);
        game.setScreen(lobbyScreen);
        lobbyScreen.setSessions(sessions.values().toArray(SessionData[]::new));
        return Optional.empty();
    }
}